package myPlayStationFriends_testing;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Formatter;

import org.junit.Assert;
import org.junit.Test;

import myPlayStationFriends.BinarySearchTree;
import myPlayStationFriends.Game;
import myPlayStationFriends.Trophy;
import myPlayStationFriends.Trophy.Rank;
import myPlayStationFriends.user;

public class BSTTester {

	@Test(expected=IllegalArgumentException.class)
	public void beFriendNullArg() {
		BinarySearchTree BST=new BinarySearchTree();
		BST.beFriend(null);
	}
	@Test
   public void beFriendDuplicate() {
	   BinarySearchTree BST=new BinarySearchTree();
	   user u=new user();
	   u.setUsername("Peter");
	   user u1=new user();
	   u1.setUsername("Peter");
	   BST.beFriend(u);
	   BST.beFriend(u1);
	   Assert.assertEquals(1, BST.countNodes());;
   }
	@Test
	   public void beFriend() {
		   BinarySearchTree BST=new BinarySearchTree();
		   user u=new user();
		   u.setUsername("Peter");
		   user u1=new user();
		   u1.setUsername("Dave");
		   BST.beFriend(u);
		   BST.beFriend(u1);
		   Assert.assertEquals(2, BST.countNodes());;
	   }
	@Test(expected=IllegalArgumentException.class)
	   public void deFriendNullArgs() {
		   BinarySearchTree BST=new BinarySearchTree();
		   user u=null;
		   BST.deFriend(null);
	   }
	@Test
	 public void deFriendNonExistent() {
		   BinarySearchTree BST=new BinarySearchTree();
		   user u=new user();
		   u.setUsername("Peter");
		   user u1=new user();
		   u1.setUsername("Dave");
		   user u2=new user();
		   u2.setUsername("John");
		   BST.beFriend(u);
		   BST.beFriend(u1);
		   
		   Assert.assertEquals(BST.deFriend(u2), BST.search(u2));
	   }
	@Test
	 public void deFriend() {
		   BinarySearchTree BST=new BinarySearchTree();
		   user u=new user();
		   u.setUsername("Peter");
		   user u1=new user();
		   u1.setUsername("Dave");
		  
		   BST.beFriend(u);
		   BST.beFriend(u1);
		   BST.deFriend(u);
		   Assert.assertEquals(1,BST.countNodes());
	   }
	@Test(expected=IllegalArgumentException.class)
	 public void countBetterPlayerNullArgs() {
		   BinarySearchTree BST=new BinarySearchTree();
		   user u=new user();
		   u.setUsername("Peter");
		   user u1=new user();
		   u1.setUsername("Dave");
		  
		   BST.beFriend(u);
		   BST.beFriend(u1);
		   BST.countBetterPlayers(null);
	   }
	@Test
	 public void countBetterPlayerNonExistent() {
		   BinarySearchTree BST=new BinarySearchTree();
		   user u=new user();
		   u.setUsername("Peter");
		   user u1=new user();
		   u1.setUsername("Dave");
		   user u3=new user();
		   u3.setUsername("Baker");
		  
		   BST.beFriend(u);
		   BST.beFriend(u1);
		   BST.countBetterPlayers(u3);
		   Assert.assertEquals(0,BST.countBetterPlayers(u3));

	   }
	@Test
	public void countBetterPlayers() {
		 BinarySearchTree BST=new BinarySearchTree();
		 ArrayList<Trophy> trophies=new ArrayList<Trophy>();
		 ArrayList<Trophy> trophies1=new ArrayList<Trophy>();
		 Game g=new Game();
		 g.setName("The two thrones");
		 user u=new user();
		   u.setUsername("Peter");
		   Trophy tr=new Trophy();
		   tr.setGame(g);
		   tr.setRank(Rank.GOLD);
		   trophies.add(tr);
		   u.setTrophies(trophies);
		   user u1=new user();
		   u1.setUsername("Dave");
		   Trophy tr1=new Trophy();
		   tr1.setGame(g);
		   tr1.setRank(Rank.SILVER);
		   trophies1.add(tr1);
		   u1.setTrophies(trophies1);
		   BST.beFriend(u);
		   BST.beFriend(u1);
		   BST.countBetterPlayers(u1);
		   Assert.assertEquals(1,BST.countBetterPlayers(u1));
		
	}
	@Test
	public void mostPlatinums() {
		 BinarySearchTree BST=new BinarySearchTree();
		 ArrayList<Trophy> trophies=new ArrayList<Trophy>();
		 ArrayList<Trophy> trophies1=new ArrayList<Trophy>();
		 Game g=new Game();
		 g.setName("The two thrones");
		 user u=new user();
		   u.setUsername("Peter");
		   Trophy tr=new Trophy();
		   tr.setGame(g);
		   tr.setRank(Rank.GOLD);
		   trophies.add(tr);
		   u.setTrophies(trophies);
		   user u1=new user();
		   u1.setUsername("Dave");
		   Trophy tr1=new Trophy();
		   tr1.setGame(g);
		   tr1.setRank(Rank.SILVER);
		   trophies1.add(tr1);
		   u1.setTrophies(trophies1);
		   BST.beFriend(u);
		   BST.beFriend(u1);
		   BST.countBetterPlayers(u1);
		   Assert.assertEquals(null,BST.mostPlatinums());
		
	}
	@Test(expected=IllegalArgumentException.class)
	public void addGameNullArgument() {
		 BinarySearchTree BST=new BinarySearchTree();
		 ArrayList<Trophy> trophies=new ArrayList<Trophy>();
		 ArrayList<Trophy> trophies1=new ArrayList<Trophy>();
		 Game g=new Game();
		 g.setName("The two thrones");
		 user u=new user();
		   u.setUsername("Peter");
		   Trophy tr=new Trophy();
		   tr.setGame(g);
		   tr.setRank(Rank.GOLD);
		   trophies.add(tr);
		   u.setTrophies(trophies);
		   BST.addGame("Peter",null);
		   
		   BST.beFriend(u);
		   
		   
		
	}
	@Test
	public void addGame() {
		 BinarySearchTree BST=new BinarySearchTree();
		 ArrayList<Trophy> trophies=new ArrayList<Trophy>();
		 ArrayList<Trophy> trophies1=new ArrayList<Trophy>();
		 Game game=new Game();
		 game.setName("The two thrones");
		 user u=new user();
		   u.setUsername("Peter");
		   Trophy tr=new Trophy();
		   tr.setGame(game);
		   tr.setRank(Rank.GOLD);
		   trophies.add(tr);
		   u.setTrophies(trophies);
		   BST.beFriend(u);
		   BST.addGame("Peter",game);
			Assert.assertEquals("\""+game.getName()+"\""+", released on: "+game.getFmt().format("%tb",game.getReleased())+" "+game.getReleased().get(Calendar.DATE)+","+game.getReleased().get(Calendar.YEAR),u.getGames().getGame("The two thrones").ToString());

		   
		   
		   
		
			
	}
	@Test(expected=IllegalArgumentException.class)
	public void addTrophyNullArgument() {
		 BinarySearchTree BST=new BinarySearchTree();
		 ArrayList<Trophy> trophies=new ArrayList<Trophy>();
		 ArrayList<Trophy> trophies1=new ArrayList<Trophy>();
		 Game g=new Game();
		 g.setName("The two thrones");
		 user u=new user();
		   u.setUsername("Peter");
		   Trophy tr=new Trophy();
		   tr.setGame(g);
		   tr.setRank(Rank.GOLD);
		   trophies.add(tr);
		   u.setTrophies(trophies);
		   BST.addTrophy("Peter",null);
		   
		   BST.beFriend(u);
		   
		   
		
	}
	@Test
	public void addTrophy() {
		 BinarySearchTree BST=new BinarySearchTree();
		 ArrayList<Trophy> trophies=new ArrayList<Trophy>();
		 ArrayList<Trophy> trophies1=new ArrayList<Trophy>();
		 Game g=new Game();
		 g.setName("The two thrones");
		 user u=new user();
		   u.setUsername("Peter");
		   Trophy tr=new Trophy();
		   tr.setGame(g);
		   tr.setRank(Rank.GOLD);
		   trophies.add(tr);
		   u.setTrophies(trophies);
		   BST.addTrophy("Peter",tr);
		   
		   BST.beFriend(u);
		   Formatter fmt=new Formatter();
			Assert.assertEquals("\""+tr.getName()+"\""+", rank: "+tr.getRank()+", rarity: "+tr.getRarity()+", obtained on: "+fmt.format("%tb",tr.getObtained())+" "+tr.getObtained().get(Calendar.DATE)+","+tr.getObtained().get(Calendar.YEAR),tr.ToString());

		   
	}
	
	@Test(expected=IllegalArgumentException.class)
	public void levelUpNullArguments() {
		 BinarySearchTree BST=new BinarySearchTree();
		 ArrayList<Trophy> trophies=new ArrayList<Trophy>();
		 ArrayList<Trophy> trophies1=new ArrayList<Trophy>();
		 Game g=new Game();
		 g.setName("The two thrones");
		 user u=new user();
		   u.setUsername("Peter");
		   Trophy tr=new Trophy();
		   tr.setGame(g);
		   tr.setRank(Rank.GOLD);
		   trophies.add(tr);
		   u.setTrophies(trophies);
		   BST.levelUp(null);
		   
		   BST.beFriend(u);
		  
	}
	
	@Test
	public void levelUp(){
		 BinarySearchTree BST=new BinarySearchTree();
		 ArrayList<Trophy> trophies=new ArrayList<Trophy>();
		 ArrayList<Trophy> trophies1=new ArrayList<Trophy>();
		 Game g=new Game();
		 g.setName("The two thrones");
		 user u=new user();
		   u.setUsername("Peter");
		   Trophy tr=new Trophy();
		   tr.setGame(g);
		   tr.setRank(Rank.GOLD);
		   trophies.add(tr);
		   u.setTrophies(trophies);
		   BST.beFriend(u);
		   BST.levelUp("Peter");
		   
		  
		   Assert.assertEquals(1, BST.countNodes());
		  
	}
	@Test
	public void testToString() {
		BinarySearchTree BST=new BinarySearchTree();
		 ArrayList<Trophy> trophies=new ArrayList<Trophy>();
		 ArrayList<Trophy> trophies1=new ArrayList<Trophy>();
		 Game g=new Game();
		 g.setName("The two thrones");
		 user u=new user();
		   u.setUsername("Peter");
		   Trophy tr=new Trophy();
		   tr.setGame(g);
		   tr.setRank(Rank.GOLD);
		   trophies.add(tr);
		   u.setTrophies(trophies);
		   BST.beFriend(u);
		   BST.levelUp("Peter");
		   Assert.assertEquals(1, BST.countNodes());
	}
	
	
}
